#include "Jester.h"
#include "Global.h"
#include <fstream>
using namespace std;

CJesterOptions JesterOptions;
CJester *Jester;

// InitD3D
bool CJester::InitD3D(HWND hwnd, int width, int height, bool fullscreen)
{
	Log("Initializing Direct3D...");
	// Correct Window Size
	RECT WindowNewSize = {0, 0, width, height};
	AdjustWindowRectEx(&WindowNewSize, WS_SYSMENU | WS_OVERLAPPED, 0, 0);
	MoveWindow(hwnd, 0, 0, WindowNewSize.right - WindowNewSize.left,
		                         WindowNewSize.bottom - WindowNewSize.top, FALSE);

	Log("Creating Direct3D Interface...");
	if(FAILED(pd3d = Direct3DCreate8(D3D_SDK_VERSION)))
	{
		MessageBox(hwnd, "Direct3DCreate8() failed", "Error", MB_ICONSTOP | MB_OK);
		Log("Direct3DCreate8() FAILED");
		return false;
	}

	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));

	if(fullscreen)
	{
		d3dpp.Windowed = FALSE;
		d3dpp.BackBufferWidth = width;
		d3dpp.BackBufferHeight = height;
		d3dpp.BackBufferCount = 1;
		d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
		d3dpp.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;
		d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
		d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
		d3dpp.BackBufferFormat = D3DFMT_R5G6B5;
	}
	else
	{
		D3DDISPLAYMODE d3ddm;
		if(FAILED(pd3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
		{
			Log("GetAdapterDisplayMode() FAILED");
			return false;
		}
		d3dpp.Windowed = TRUE;
		d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
		d3dpp.BackBufferFormat = d3ddm.Format;
	}

	Log("Creating Device...");
	if(FAILED(pd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                  D3DCREATE_HARDWARE_VERTEXPROCESSING ,
                                  &d3dpp, &pd3dDevice)))
	{
		Log("Hardware Vertex Processing unsupported, switching to software...");
		if(FAILED(pd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                  D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                  &d3dpp, &pd3dDevice)))
		{
			Log("Hardware Driver Failed, or unknown error, attempting to switch to reference driver...");
			if(FAILED(pd3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, hwnd,
                                  D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                  &d3dpp, &pd3dDevice)))
			{
				MessageBox(hwnd, "CreateDevice() failed", "Error", MB_ICONSTOP | MB_OK);
				Log("CreateDevice() FAILED");
				return false;
			}
		}
	} 

	Log("Initializing Engine...");
	MainWindow = hwnd;
	JesterOptions.Console = 1;
	JesterOptions.WireFrame = 0;
	JesterOptions.Failed = 0;
	TextureList = new CTextureList;
	SpriteList = new CSpriteList;

	D3DXMATRIX Ortho2D;
	D3DXMATRIX Identity;
	
	D3DXMatrixOrthoLH(&Ortho2D, width, height, 0.0f, 1.0f);
	D3DXMatrixIdentity(&Identity);

	pd3dDevice->SetTransform(D3DTS_PROJECTION, &Ortho2D);
	pd3dDevice->SetTransform(D3DTS_WORLD, &Identity);
	pd3dDevice->SetTransform(D3DTS_VIEW, &Identity);

	pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,  TRUE);
	pd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	pd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	Log("Done!");
	return true;
}

// Shutdown
void CJester::Shutdown()
{
	if(pd3dDevice)
	{
        pd3dDevice->Release();
		pd3dDevice = 0;
	}

    if(pd3d)
	{
        pd3d->Release();
		pd3d = 0;
	}
}

HSTREAM CJester::CreateSound(char* filename, bool autofree)
{
	HSTREAM newstream;
	if(autofree) newstream = BASS_StreamCreateFile(FALSE, filename, 0, 0, BASS_STREAM_AUTOFREE);
	else newstream = BASS_StreamCreateFile(FALSE, filename, 0, 0, 0); 
	
	if(!newstream)
	{
		MessageBox(g_hwnd, "Could not create newstream", "Error", MB_ICONERROR | MB_OK);
		Log("Could not create newstream");
		return 0;
	}

	return newstream;
}

void CJester::PlaySound(HSTREAM sound, bool loop)
{
	if(loop)if(!BASS_StreamPlay(sound, FALSE, BASS_SAMPLE_LOOP)) Jester->Log("Could not play sound");
	else if(!BASS_StreamPlay(sound, FALSE, 0)) Jester->Log("Could not play sound");
	if(!BASS_ChannelIsActive(sound)) Jester->Log("Playsound channel inactive");
}

bool CJester::SoundInit()
{
	if(!BASS_Init(-1,44100,0,g_hwnd)) 
	{
		int ret = MessageBox(g_hwnd, "Couldn't Init Bass.\nContinue without sound?", "Error", MB_ICONERROR | MB_YESNO);
		Jester->Log("Couldn't Init Bass");
		if(ret == IDYES) BASS_Init(-2,44100,0,g_hwnd);
		else return false;
	}
	BASS_Start();
	return true;
}

// BeginFrame
void CJester::BeginFrame()
{
	if(FAILED(pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,0), 1.0f, 0)))
	{
		Log("Clear() FAILED");
		PostQuitMessage(1);
	}
	pd3dDevice->BeginScene();	
}

// EndFrame
void CJester::EndFrame()
{
	pd3dDevice->EndScene();
	pd3dDevice->Present(NULL, NULL, NULL, NULL);
}
// Check if a file exists
bool CJester::CheckExists(char *Filename)
{
	ifstream File(Filename, ios::binary);
	if(File.fail())
	{
		File.close();
		char buffer[30];
		sprintf(buffer, "The file %s, could not be opened, check that it exists.", Filename);
		MessageBox(MainWindow, buffer, "Error", MB_ICONSTOP | MB_OK);
		Log(buffer);
		return false;
	}
	File.close();
	return true;
}

void CJester::TextOutBlack(int x, int y, char* buffer)
{
	char *text = new char[strlen(buffer)];
	for(int k = 0; k < strlen(buffer); k++)
	{
		text[k] = buffer[k];
		if((text[k] > 96) && (text[k] < 123)) text[k] -=32;
	}

	for(int i = 0; i < strlen(buffer); i++)
	{
		DrawSprite(g_FontID[(int)text[i]], x+i*8, y);
	}
}

void CJester::TextOutWhite(int x, int y, char* buffer)
{
	char *text = new char[strlen(buffer)];
	for(int k = 0; k < strlen(buffer); k++)
	{
		text[k] = buffer[k];
		if((text[k] > 96) && (text[k] < 123)) text[k] -=32;
	}

	for(int i = 0; i < strlen(buffer); i++)
	{
		DrawSprite(g_FontWhiteID[(int)text[i]], x+i*8, y);
	}
}

void CJester::Set(int option, int value)
{
	switch(option)
	{
		case JESTER_WIREFRAME:
			if(value) pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
			else pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
			JesterOptions.WireFrame = !JesterOptions.WireFrame;
			break;

		case JESTER_CONSOLE:
			JesterOptions.Console = value;
			break;

		default:
			MessageBox(MainWindow, "Unknown option sent to Set()", "Error", MB_ICONEXCLAMATION | MB_OK);
			Log("Unknown option sent to Set()");
			break;
	}
}

// Failed
bool CJester::Failed()
{
	if(JesterOptions.Failed) return true;
	else return false;
}

// Logging Functions
void CJester::Log(int Number, int SetFlag)
{
	char Flag = '\n';
	if(SetFlag == 0) Flag = ' ';
	
	ofstream File("Log.txt", ios::app);
	if(!File)
	{
		MessageBox(NULL, "Could not open Log File.", "Error", MB_ICONSTOP);
		return;
	}
	File << Number << Flag;
	File.close();
	return;
}

void CJester::Log(float Number, int SetFlag)
{
	char Flag = '\n';
	if(SetFlag == 0) Flag = ' ';
	
	ofstream File("Log.txt", ios::app);
	if(!File)
	{
		MessageBox(NULL, "Could not open Log File.", "Error", MB_ICONSTOP);
		return;
	}
	File << Number << Flag;
	File.close();
	return;
}

void CJester::LogInit()
{
	ofstream File("Log.txt"); // Trunc file
	if(!File)
	{
		MessageBox(NULL, "Could not open Log File.", "Error", MB_ICONSTOP);
		return;
	}
	File.close();
}

void CJester::Log(const char *Text, int SetFlag) // Set flag defulted to 1
{
	char Flag = '\n';
	if(SetFlag == 0) Flag = ' ';

	ofstream File("Log.txt", ios::app);
	if(!File)
	{
		MessageBox(NULL, "Could not open Log File.", "Error", MB_ICONSTOP);
		return;
	}
	File << Text << Flag;
	File.close();
	return;
}