#ifndef JESTER_ENGINE_H
#define JESTER_ENGINE_H

#include <d3d8.h>
#include <d3dx8.h>
#include <string>
#include <mmsystem.h>
#include <stdio.h>
#include "bass.h"
using namespace std;

struct CFVFVertex
{
	float x, y, z;
	DWORD color;
	float u, v;
};

struct CJesterOptions
{
	int Console;
	int WireFrame;
	int Failed;
};
extern struct CJesterOptions JesterOptions;

#define JESTER_WIREFRAME	1
#define JESTER_CONSOLE		2

#include "TextureManager.h"
#include "SpriteManager.h"

extern class CJester *Jester;

class CJester
{
public:
	friend class CTextureNode;
	friend class CSpriteNode;

	bool InitD3D(HWND hwnd, int width, int height, bool fullscreen);	
	void Shutdown();

	void BeginFrame();
	void EndFrame();

	bool SoundInit();
	HSTREAM CreateSound(char* filename, bool autofree);
	void PlaySound(HSTREAM sound, bool loop);
	void FreeSound(HSTREAM sound){BASS_StreamFree(sound);}
	void PauseSounds(){BASS_Pause();}

	int CreateSprite(char* Filename, int x1, int y1, int x2, int y2);
	void DrawSprite(int ID, int x, int y);
	void DrawScaledSprite(int ID, int x, int y, int scaledPercent);
	void DrawRotatedSprite(int ID, int x, int y, float angle, char dimension);

	void TextOutBlack(int x, int y, char* text);
	void TextOutWhite(int x, int y, char* text);

	void Set(int option, int value);
	bool Failed();

	void LogInit();
	static void Log(const char *Text, int SetFlag = 1);
	static void Log(int Number, int SetFlag = 1);
	static void Log(float Number, int SetFlag = 1);

	~CJester(){Shutdown();}
	
private:
	bool CheckExists(char *Filename);

	IDirect3D8 *pd3d;
	IDirect3DDevice8 *pd3dDevice;

	CTextureList *TextureList;
	CSpriteList *SpriteList;

	HWND MainWindow;
};

#endif


